=== modified file 'bzrlib/plugins/weave_fmt/workingtree.py'
--- bzrlib/plugins/weave_fmt/workingtree.py	2011-06-28 11:29:03 +0000
+++ bzrlib/plugins/weave_fmt/workingtree.py	2011-08-30 09:29:03 +0000
@@ -17,13 +17,16 @@
 """Weave-era working tree objects."""
 
 from cStringIO import StringIO
+import errno
 
 from bzrlib import (
     conflicts as _mod_conflicts,
     errors,
+    hashcache,
     inventory,
     osutils,
     revision as _mod_revision,
+    trace,
     transform,
     xml5,
     )
@@ -143,8 +146,29 @@
      - uses the branch last-revision.
     """
 
-    def __init__(self, *args, **kwargs):
-        super(WorkingTree2, self).__init__(*args, **kwargs)
+    def __init__(self, basedir, *args, **kwargs):
+        super(WorkingTree2, self).__init__(basedir, *args, **kwargs)
+
+        # update the whole cache up front and write to disk if anything changed;
+        # in the future we might want to do this more selectively
+        # two possible ways offer themselves : in self._unlock, write the cache
+        # if needed, or, when the cache sees a change, append it to the hash
+        # cache file, and have the parser take the most recent entry for a
+        # given path only.
+        wt_trans = self.bzrdir.get_workingtree_transport(None)
+        cache_filename = wt_trans.local_abspath('stat-cache')
+        self._hashcache = hashcache.HashCache(basedir, cache_filename,
+            self.bzrdir._get_file_mode(),
+            self._content_filter_stack_provider())
+        hc = self._hashcache
+        hc.read()
+        # is this scan needed ? it makes things kinda slow.
+        #hc.scan()
+
+        if hc.needs_write:
+            trace.mutter("write hc")
+            hc.write()
+
         # WorkingTree2 has more of a constraint that self._inventory must
         # exist. Because this is an older format, we don't mind the overhead
         # caused by the extra computation here.
@@ -158,6 +182,13 @@
         """Return the references needed to perform a check of this tree."""
         return [('trees', self.last_revision())]
 
+
+    @needs_read_lock
+    def get_file_sha1(self, file_id, path=None, stat_value=None):
+        if not path:
+            path = self._inventory.id2path(file_id)
+        return self._hashcache.get_sha1(path, stat_value)
+
     def lock_tree_write(self):
         """See WorkingTree.lock_tree_write().
 
@@ -175,6 +206,20 @@
             self.branch.unlock()
             raise
 
+    def _write_hashcache_if_dirty(self):
+        """Write out the hashcache if it is dirty."""
+        if self._hashcache.needs_write:
+            try:
+                self._hashcache.write()
+            except OSError, e:
+                if e.errno not in (errno.EPERM, errno.EACCES):
+                    raise
+                # TODO: jam 20061219 Should this be a warning? A single line
+                #       warning might be sufficient to let the user know what
+                #       is going on.
+                trace.mutter('Could not write hashcache for %s\nError: %s',
+                              self._hashcache.cache_file_name(), e)
+
     def unlock(self):
         # we share control files:
         if self._control_files._lock_count == 3:

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2011-08-29 21:29:02 +0000
+++ bzrlib/workingtree.py	2011-08-30 09:29:03 +0000
@@ -54,7 +54,6 @@
     generate_ids,
     globbing,
     graph as _mod_graph,
-    hashcache,
     ignores,
     inventory,
     merge,
@@ -1030,7 +1029,6 @@
             stream.write(bytes)
         finally:
             stream.close()
-        # TODO: update the hashcache here ?
 
     def extras(self):
         """Yield all unversioned files in this WorkingTree.
@@ -1781,25 +1779,6 @@
             branch=branch, _control_files=_control_files, _internal=_internal,
             _format=_format, _bzrdir=_bzrdir)
 
-        # update the whole cache up front and write to disk if anything changed;
-        # in the future we might want to do this more selectively
-        # two possible ways offer themselves : in self._unlock, write the cache
-        # if needed, or, when the cache sees a change, append it to the hash
-        # cache file, and have the parser take the most recent entry for a
-        # given path only.
-        wt_trans = self.bzrdir.get_workingtree_transport(None)
-        cache_filename = wt_trans.local_abspath('stat-cache')
-        self._hashcache = hashcache.HashCache(basedir, cache_filename,
-            self.bzrdir._get_file_mode(),
-            self._content_filter_stack_provider())
-        hc = self._hashcache
-        hc.read()
-        # is this scan needed ? it makes things kinda slow.
-        #hc.scan()
-
-        if hc.needs_write:
-            mutter("write hc")
-            hc.write()
         self._detect_case_handling()
 
         if _inventory is None:
@@ -1837,20 +1816,6 @@
 
         self._setup_directory_is_tree_reference()
 
-    def _write_hashcache_if_dirty(self):
-        """Write out the hashcache if it is dirty."""
-        if self._hashcache.needs_write:
-            try:
-                self._hashcache.write()
-            except OSError, e:
-                if e.errno not in (errno.EPERM, errno.EACCES):
-                    raise
-                # TODO: jam 20061219 Should this be a warning? A single line
-                #       warning might be sufficient to let the user know what
-                #       is going on.
-                mutter('Could not write hashcache for %s\nError: %s',
-                              self._hashcache.cache_file_name(), e)
-
     def _serialize(self, inventory, out_file):
         xml5.serializer_v5.write_inventory(self._inventory, out_file,
             working=True)
@@ -2160,12 +2125,6 @@
             mode=self.bzrdir._get_file_mode())
         self._inventory_is_modified = False
 
-    @needs_read_lock
-    def get_file_sha1(self, file_id, path=None, stat_value=None):
-        if not path:
-            path = self._inventory.id2path(file_id)
-        return self._hashcache.get_sha1(path, stat_value)
-
     def get_file_mtime(self, file_id, path=None):
         """See Tree.get_file_mtime."""
         if not path:

=== modified file 'bzrlib/workingtree_3.py'
--- bzrlib/workingtree_3.py	2011-06-28 11:29:03 +0000
+++ bzrlib/workingtree_3.py	2011-08-30 09:29:03 +0000
@@ -18,11 +18,15 @@
 
 """
 
+import errno
+
 from bzrlib import (
     bzrdir,
     errors,
+    hashcache,
     inventory,
     revision as _mod_revision,
+    trace,
     transform,
     )
 from bzrlib.decorators import (
@@ -46,6 +50,29 @@
     This is new in bzr 0.8
     """
 
+    def __init__(self, basedir='.', *args, **kwargs):
+        super(WorkingTree3, self).__init__(basedir, *args, **kwargs)
+
+        # update the whole cache up front and write to disk if anything changed;
+        # in the future we might want to do this more selectively
+        # two possible ways offer themselves : in self._unlock, write the cache
+        # if needed, or, when the cache sees a change, append it to the hash
+        # cache file, and have the parser take the most recent entry for a
+        # given path only.
+        wt_trans = self.bzrdir.get_workingtree_transport(None)
+        cache_filename = wt_trans.local_abspath('stat-cache')
+        self._hashcache = hashcache.HashCache(basedir, cache_filename,
+            self.bzrdir._get_file_mode(),
+            self._content_filter_stack_provider())
+        hc = self._hashcache
+        hc.read()
+        # is this scan needed ? it makes things kinda slow.
+        #hc.scan()
+
+        if hc.needs_write:
+            trace.mutter("write hc")
+            hc.write()
+
     @needs_read_lock
     def _last_revision(self):
         """See Mutable.last_revision."""
@@ -85,6 +112,26 @@
         finally:
             self.branch.unlock()
 
+    def _write_hashcache_if_dirty(self):
+        """Write out the hashcache if it is dirty."""
+        if self._hashcache.needs_write:
+            try:
+                self._hashcache.write()
+            except OSError, e:
+                if e.errno not in (errno.EPERM, errno.EACCES):
+                    raise
+                # TODO: jam 20061219 Should this be a warning? A single line
+                #       warning might be sufficient to let the user know what
+                #       is going on.
+                trace.mutter('Could not write hashcache for %s\nError: %s',
+                              self._hashcache.cache_file_name(), e)
+
+    @needs_read_lock
+    def get_file_sha1(self, file_id, path=None, stat_value=None):
+        if not path:
+            path = self._inventory.id2path(file_id)
+        return self._hashcache.get_sha1(path, stat_value)
+
 
 class WorkingTreeFormat3(WorkingTreeFormat):
     """The second working tree format updated to record a format marker.

